using System;
using System.ComponentModel;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using ASP = System.Web.UI.WebControls;

namespace Anthem
{
    public class Debugger : ASP.WebControl, INamingContainer, IUpdatableControl
    {
        protected override void CreateChildControls()
        {
            Controls.Add(new LiteralControl("<div><fieldset><legend>Anthem Debugger</legend>"));
            AddCheckBox("RequestText");
            AddCheckBox("ResponseText");
            AddCheckBox("Errors");
            Controls.Add(new LiteralControl("</fieldset></div>"));
        }

        void AddCheckBox(string option)
        {
            HtmlInputCheckBox cb = new HtmlInputCheckBox();
            Controls.Add(cb);
            cb.ID = option;
            cb.Attributes["onchange"] = string.Format("AnthemDebugger_Debug{0} = this.checked", option);
            Controls.Add(new LiteralControl(string.Format(" {0} ", option)));
        }

        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);

            Anthem.Manager.Register(this);

            if (Page.IsPostBack)
            {
                HtmlInputCheckBox requestText = FindControl("RequestText") as HtmlInputCheckBox;
                if (requestText != null)
                {
                    DebugRequestText = requestText.Checked;
                }

                HtmlInputCheckBox responseText = FindControl("ResponseText") as HtmlInputCheckBox;
                if (responseText != null)
                {
                    DebugResponseText = responseText.Checked;
                }

                HtmlInputCheckBox errors = FindControl("Errors") as HtmlInputCheckBox;
                if (errors != null)
                {
                    DebugErrors = errors.Checked;
                }
            }
        }

        protected override void OnPreRender(EventArgs e)
        {
            base.OnPreRender(e);

            HtmlInputCheckBox requestText = FindControl("RequestText") as HtmlInputCheckBox;
            if (requestText != null)
            {
                requestText.Checked = DebugRequestText;
            }

            HtmlInputCheckBox responseText = FindControl("ResponseText") as HtmlInputCheckBox;
            if (responseText != null)
            {
                responseText.Checked = DebugResponseText;
            }

            HtmlInputCheckBox errors = FindControl("Errors") as HtmlInputCheckBox;
            if (errors != null)
            {
                errors.Checked = DebugErrors;
            }

            string script = string.Format(
                @"<script language='javascript' type='text/javascript'>
var AnthemDebugger_DebugRequestText = {0};
var AnthemDebugger_DebugResponseText = {1};
var AnthemDebugger_DebugErrors = {2};
function Anthem_DebugRequestText(text) {{
    if (AnthemDebugger_DebugRequestText) {{
        alert('Anthem Debugger (RequestText):\n\n' + text);
    }}
}}
function Anthem_DebugResponseText(text) {{
    if (AnthemDebugger_DebugResponseText) {{
        alert('Anthem Debugger (ResponseText):\n\n' + text);
    }}
}}
function Anthem_DebugError(text) {{
    if (AnthemDebugger_DebugErrors) {{
        alert('Anthem Debugger (Error):\n\n' + text);
    }}
}}
</script>",
                DebugRequestText ? "true" : "false",
                DebugResponseText ? "true" : "false",
                DebugErrors ? "true" : "false"
            );

#if V2
            Page.ClientScript.RegisterClientScriptBlock(typeof(Debugger), "script", script);
#else
            Page.RegisterClientScriptBlock(typeof(Debugger).FullName, script);
#endif
        }

        protected override void Render(HtmlTextWriter writer)
        {
            Anthem.Manager.WriteBeginControlMarker(writer, "div", this);
            if (Visible)
            {
                base.Render(writer);
            }
            Anthem.Manager.WriteEndControlMarker(writer, "div", this);
        }

        public bool DebugRequestText
        {
            get
            {
                return ViewState["DebugRequestText"] != null
                    ? (bool)ViewState["DebugRequestText"]
                    : false;
            }

            set
            {
                if (IsTrackingViewState && Anthem.Manager.IsCallBack && DebugRequestText != value)
                {
                    Anthem.Manager.AddScriptForClientSideEval(
                        string.Format(
                        "AnthemDebugger_DebugRequestText = {0}",
                        value ? "true" : "false"));
                }
                ViewState["DebugRequestText"] = value;
            }
        }

        public bool DebugResponseText
        {
            get
            {
                return ViewState["DebugResponseText"] != null
                    ? (bool)ViewState["DebugResponseText"]
                    : false;
            }

            set
            {
                if (IsTrackingViewState && Anthem.Manager.IsCallBack && DebugResponseText != value)
                {
                    Anthem.Manager.AddScriptForClientSideEval(
                        string.Format(
                            "AnthemDebugger_DebugResponseText = {0}",
                            value ? "true" : "false"));
                }
                ViewState["DebugResponseText"] = value;
            }
        }

        public bool DebugErrors
        {
            get
            {
                return ViewState["DebugErrors"] != null
                    ? (bool)ViewState["DebugErrors"]
                    : false;
            }

            set
            {
                if (IsTrackingViewState && Anthem.Manager.IsCallBack && DebugErrors != value)
                {
                    Anthem.Manager.AddScriptForClientSideEval(
                        string.Format(
                        "AnthemDebugger_DebugErrors = {0}",
                        value ? "true" : "false"));
                }
                ViewState["DebugErrors"] = value;
            }
        }

        #region Common Anthem control members (copied into each control)

#if V2
        public override void RenderControl(HtmlTextWriter writer)
        {
            base.Visible = true;
            base.RenderControl(writer);
        }
#endif

        public override bool Visible
        {
            get
            {
#if !V2
                bool DesignMode = this.Context == null;
#endif
                return Anthem.Manager.GetControlVisible(this, ViewState, DesignMode);
            }
            set { Anthem.Manager.SetControlVisible(ViewState, value); }
        }

        private bool _updateAfterCallBack = false;

        [DefaultValue(false)]
        public bool AutoUpdateAfterCallBack
        {
            get
            {
                if (ViewState["AutoUpdateAfterCallBack"] == null)
                    return false;
                else
                    return (bool)ViewState["AutoUpdateAfterCallBack"];
            }
            set
            {
                if (value) UpdateAfterCallBack = true;
                ViewState["AutoUpdateAfterCallBack"] = value;
            }
        }

        [Browsable(false), DefaultValue(false)]
        public bool UpdateAfterCallBack
        {
            get { return _updateAfterCallBack; }
            set { _updateAfterCallBack = value; }
        }

        [
        Category("Misc"),
        Description("Fires before the control is rendered with updated values.")
        ]
        public event EventHandler PreUpdate
        {
            add { Events.AddHandler(EventPreUpdateKey, value); }
            remove { Events.RemoveHandler(EventPreUpdateKey, value); }
        }
        private static readonly object EventPreUpdateKey = new object();

        public virtual void OnPreUpdate()
        {
            EventHandler EditHandler = (EventHandler)Events[EventPreUpdateKey];
            if (EditHandler != null)
                EditHandler(this, EventArgs.Empty);
        }

        [DefaultValue(true)]
        public bool EnableCallBack
        {
            get
            {
                if (ViewState["EnableCallBack"] == null)
                    return true;
                else
                    return (bool)ViewState["EnableCallBack"];
            }
            set
            {
                ViewState["EnableCallBack"] = value;
            }
        }

        #endregion
    }
}
