using System;
using System.ComponentModel;
using System.Web.UI;
using ASP = System.Web.UI.WebControls;

namespace Anthem
{
    public class Timer : Control, IUpdatableControl
    {
        #region Unique Anthem control code

        [DefaultValue(false)]
        public bool Enabled
        {
            get
            {
                if (null == ViewState["Enabled"])
                    return false;
                else
                    return (bool)ViewState["Enabled"];
            }
            set
            {
                if (IsTrackingViewState && !Enabled && value)
                {
                    _addTimeoutScript = true;
                }
                ViewState["Enabled"] = value;
            }
        }

        [DefaultValue(1000)]
        public int Interval
        {
            get
            {
                if (null == ViewState["Interval"])
                    return 1000;
                else
                    return (int)ViewState["Interval"];
            }
            set { ViewState["Interval"] = value; }
        }

        [Anthem.Method]
        public void DoTick()
        {
            OnTick(EventArgs.Empty);
            _addTimeoutScript = true;
        }

        public event EventHandler Tick;
        protected virtual void OnTick(EventArgs e)
        {
            if (Tick != null)
            {
                Tick(this, e);
            }
        }

        private bool _addTimeoutScript = false;
        protected override void OnPreRender(EventArgs e)
        {
            base.OnPreRender(e);
            if (Enabled)
            {
                if (!Anthem.Manager.IsCallBack)
                {
                    Controls.Add(new LiteralControl(
                        string.Format(
                            "<script type='text/javascript'>Anthem_AddEvent(window, 'load', function() {{ {0} }})</script>",
                            GetSetTimeoutScript()
                        )
                    ));
                }
                else
                {
                    if (_addTimeoutScript)
                    {
                        Anthem.Manager.AddScriptForClientSideEval(GetSetTimeoutScript());
                    }
                }
            }
        }

        private const string parentTagName = "span";
        protected override void Render(HtmlTextWriter writer)
        {
#if !V2
            bool DesignMode = this.Context == null;
#endif
            if (!DesignMode)
            {
                Anthem.Manager.WriteBeginControlMarker(writer, parentTagName, this);
            }
            if (Visible)
            {
                base.Render(writer);
            }
            if (!DesignMode)
            {
                Anthem.Manager.WriteEndControlMarker(writer, parentTagName, this);
            }
        }

        #endregion

        #region IUpdatableControl implementation

        [DefaultValue(false)]
        public bool AutoUpdateAfterCallBack
        {
            get
            {
                if (ViewState["AutoUpdateAfterCallBack"] == null)
                    return false;
                else
                    return (bool)ViewState["AutoUpdateAfterCallBack"];
            }
            set
            {
                if (value) UpdateAfterCallBack = true;
                ViewState["AutoUpdateAfterCallBack"] = value;
            }
        }

        private bool _updateAfterCallBack = false;

        [Browsable(false), DefaultValue(false)]
        public bool UpdateAfterCallBack
        {
            get { return _updateAfterCallBack; }
            set { _updateAfterCallBack = value; }
        }

        [
        Category("Misc"),
        Description("Fires before the control is rendered with updated values.")
        ]
        public event EventHandler PreUpdate
        {
            add { Events.AddHandler(EventPreUpdateKey, value); }
            remove { Events.RemoveHandler(EventPreUpdateKey, value); }
        }
        private static readonly object EventPreUpdateKey = new object();

        public virtual void OnPreUpdate()
        {
            EventHandler EditHandler = (EventHandler)Events[EventPreUpdateKey];
            if (EditHandler != null)
                EditHandler(this, EventArgs.Empty);
        }

        #endregion

        #region Common Anthem control code

        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);
            Anthem.Manager.Register(this);
        }

#if V2
        public override void RenderControl(HtmlTextWriter writer)
        {
            base.Visible = true;
            base.RenderControl(writer);
        }
#endif

        public override bool Visible
        {
            get
            {
#if !V2
                bool DesignMode = this.Context == null;
#endif
                return Anthem.Manager.GetControlVisible(this, ViewState, DesignMode);
            }
            set { Anthem.Manager.SetControlVisible(ViewState, value); }
        }

        #endregion

        #region Private Methods

        private string GetSetTimeoutScript()
        {
            return string.Format(
                "setTimeout(function() {{ Anthem_InvokeControlMethod('{0}', 'DoTick', [], function() {{ }}, null) }}, {1})",
                ClientID,
                Interval
            );
        }

        #endregion
    }
}
