using System;
using System.ComponentModel;
using System.Drawing;
using System.Web.UI;
using ASP = System.Web.UI.WebControls;

namespace Anthem
{
    [ToolboxBitmap(typeof(ASP.ValidationSummary))]
    public class ValidationSummary : ASP.ValidationSummary, IUpdatableControl
    {
        #region Unique Anthem control code

        protected override void OnInit(EventArgs e)
        {
            base.OnInit(e);
            this.Load += new EventHandler(Validator_Load);
        }

        private bool _initialVisibility = false;
        void Validator_Load(object sender, EventArgs e)
        {
            _initialVisibility = this.Visible;
        }

        /// <summary>
        /// If the visibility of this validator has changed during this callback,
        /// then force Anthem.Manager to include the page scripts (which include
        /// the validation scripts).
        /// 
        /// SaveViewState is overridden because it is called whether the control
        /// is visible or not.
        /// </summary>
        /// <returns></returns>
        protected override object SaveViewState()
        {
            if (Anthem.Manager.IsCallBack && _initialVisibility != this.Visible)
                Anthem.Manager.IncludePageScripts = true;
            return base.SaveViewState();
        }

        [DefaultValue(true)]
        public bool EnableCallBackScript
        {
            get
            {
                if (null == ViewState["EnableCallBackScript"])
                    return true;
                else
                    return (bool)ViewState["EnableCallBackScript"];
            }
            set { ViewState["EnableCallBackScript"] = value; }
        }

        /// <summary>
        /// If ShowMessageBox=true, use Anthem.Manager to display the validation
        /// summary in a javascript alert.
        /// </summary>
        /// <param name="e"></param>
        protected override void OnPreRender(EventArgs e)
        {
            base.OnPreRender(e);

            bool isValid = true;
            foreach (IValidator validator in Page.Validators)
                isValid &= validator.IsValid;

            if (Anthem.Manager.IsCallBack
              && !isValid
              && this.ShowMessageBox
              && this.EnableCallBackScript)
            {
                string s = string.Empty;
                if (this.HeaderText != null && this.HeaderText != string.Empty)
                    s += this.HeaderText + "\\n";
                int count = 0;
                foreach (ASP.BaseValidator validator in Page.Validators)
                {

#if V2
                    if (
                      ((
                        (this.ValidationGroup == null || this.ValidationGroup == string.Empty)
                        && (validator.ValidationGroup == null || validator.ValidationGroup == string.Empty)
                      )
                      ||
                      (
                        (this.ValidationGroup != null && this.ValidationGroup != string.Empty)
                        && (validator.ValidationGroup != null && validator.ValidationGroup == this.ValidationGroup)
                      ))
                      && !validator.IsValid
                      && validator.ErrorMessage != null
                      && validator.ErrorMessage != string.Empty)
#else
                    if (!validator.IsValid 
                        && validator.ErrorMessage != null 
                        && validator.ErrorMessage != string.Empty)
#endif

                    {
                        count++;
                        switch (this.DisplayMode)
                        {
                            case System.Web.UI.WebControls.ValidationSummaryDisplayMode.List:
                                s += validator.ErrorMessage + "\\n";
                                break;
                            case System.Web.UI.WebControls.ValidationSummaryDisplayMode.BulletList:
                            default:
                                s += "  - " + validator.ErrorMessage + "\\n";
                                break;
                            case System.Web.UI.WebControls.ValidationSummaryDisplayMode.SingleParagraph:
                                s += validator.ErrorMessage + " ";
                                break;
                        }
                    }
                }
                if (count > 0)
                    Anthem.Manager.AddScriptForClientSideEval("alert('" + s + "');");
            }
        }

        private const string parentTagName = "span";
        protected override void Render(HtmlTextWriter writer)
        {
#if !V2
            bool DesignMode = this.Context == null;
#endif
            if (!DesignMode)
            {
                Anthem.Manager.WriteBeginControlMarker(writer, parentTagName, this);
            }
            if (Visible)
            {
                base.Render(writer);
            }
            if (!DesignMode)
            {
                Anthem.Manager.WriteEndControlMarker(writer, parentTagName, this);
            }
        }

        #endregion

        #region IUpdatableControl implementation

        [DefaultValue(false)]
        public bool AutoUpdateAfterCallBack
        {
            get
            {
                if (ViewState["AutoUpdateAfterCallBack"] == null)
                    return false;
                else
                    return (bool)ViewState["AutoUpdateAfterCallBack"];
            }
            set
            {
                if (value) UpdateAfterCallBack = true;
                ViewState["AutoUpdateAfterCallBack"] = value;
            }
        }

        private bool _updateAfterCallBack = false;

        [Browsable(false), DefaultValue(false)]
        public bool UpdateAfterCallBack
        {
            get { return _updateAfterCallBack; }
            set { _updateAfterCallBack = value; }
        }

        [
        Category("Misc"),
        Description("Fires before the control is rendered with updated values.")
        ]
        public event EventHandler PreUpdate
        {
            add { Events.AddHandler(EventPreUpdateKey, value); }
            remove { Events.RemoveHandler(EventPreUpdateKey, value); }
        }
        private static readonly object EventPreUpdateKey = new object();

        public virtual void OnPreUpdate()
        {
            EventHandler EditHandler = (EventHandler)Events[EventPreUpdateKey];
            if (EditHandler != null)
                EditHandler(this, EventArgs.Empty);
        }

        #endregion

        #region Common Anthem control code

        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);
            Anthem.Manager.Register(this);
        }

#if V2
        public override void RenderControl(HtmlTextWriter writer)
        {
            base.Visible = true;
            base.RenderControl(writer);
        }
#endif

        public override bool Visible
        {
            get
            {
#if !V2
                bool DesignMode = this.Context == null;
#endif
                return Anthem.Manager.GetControlVisible(this, ViewState, DesignMode);
            }
            set { Anthem.Manager.SetControlVisible(ViewState, value); }
        }

        #endregion
    }
}
